capture program drop olssim

//program drop olssim
program olssim, rclass
  version 10.0
  syntax [, obs(integer 1)]
  drop _all
  set obs `obs'
  tempvar t
  gen t=_n
  tsset t
  //generate "stochastic trend or unobserved component" 
  gen xi=invnorm(uniform())
  gen alpha=xi if tin(1,1)
  replace alpha=l.alpha+xi if tin(2,`obs')

  gen eps_c = 0.007926*invnorm(uniform())
  gen eps_y = 0.01502*invnorm(uniform())
  gen eps_i = 0.045238*invnorm(uniform())

  gen eta_c = eps_c if tin(1,1)
  replace eta_c = 0.028599*l.eta_c+eps_y if tin(2,2)
  replace eta_c = 0.028599*l.eta_c+0.336564*l2.eta_c+eps_y if tin(3,3)
  replace eta_c = 0.028599*l.eta_c+0.336564*l2.eta_c-0.009532*l3.eta_c+eps_c if tin(4,4)
  replace eta_c = 0.028599*l.eta_c+0.336564*l2.eta_c-0.009532*l3.eta_c-0.172802*l4.eta_c+eps_c if tin(5,`obs')

  gen eta_y = eps_y if tin(1,1)
  replace eta_y = 0.816491*l.eta_y+eps_y if tin(2,`obs')
  replace eta_y = 0.816491*l.eta_y+0.231732*l2.eta_y+eps_y if tin(3,3)
  replace eta_y = 0.816491*l.eta_y+0.231732*l2.eta_y-0.188353*l3.eta_y+eps_y if tin(4,`obs')
  
  gen eta_i=eps_i if tin(1,1)
  replace eta_i = 1.028466*l.eta_i+eps_i if tin(2,2)
  replace eta_i = 1.028466*l.eta_i-0.043786*l2.eta_i+eps_i if tin(3,3)
  replace eta_i = 1.028466*l.eta_i-0.043786*l2.eta_i-0.163936*l3.eta_i+eps_i if tin(4,`obs')
  
  gen c = 0.0082653*alpha+eta_c 
  gen y = 0.0096193*alpha+eta_y 
  gen i = 0.0127505*alpha+eta_i 
  
  constraint define 1 [tfp]L.tfp = 1
  sspace (tfp L.tfp, state noconstant) (y tfp, nocons ) ///
  (c tfp, noconstant) (i tfp, nocons),    ///
  constraints(1) covstate(identity) iterate(300)  //
  
  predict alpha_hat, state equation(tfp)
  gen y_trend=_b[y: tfp]*alpha_hat
  gen c_trend=_b[c: tfp]*alpha_hat
  gen i_trend=_b[i: tfp]*alpha_hat
  gen con=`e(converged)'
  gen se_by=_se[y:tfp]
  gen se_bc=_se[c:tfp]
  gen se_bi=_se[i:tfp]
  gen se_y=_b[var(y):_cons]
  gen se_c=_b[var(c):_cons]
  gen se_i=_b[var(i):_cons]
  
  foreach var of varlist y c i {
  gen `var'_trans=`var'-`var'_trend
  gen rmse1_`var'=(`var'_trans-eta_`var')^2
  egen rmse2_`var'=sum(rmse1_`var')
  gen rmse3_`var'=rmse2_`var'/(`obs'-2)
  gen rmse_`var'=rmse3_`var'^0.5
  }
  
  return scalar conv=con
  return scalar seby=se_by
  return scalar sebc=se_bc
  return scalar sebi=se_bi
  return scalar sey=se_y
  return scalar sec=se_c
  return scalar sei=se_i  
  return scalar rmsey=rmse_y
  return scalar rmsec=rmse_c
  return scalar rmsei=rmse_i
  return scalar bc=_b[c: tfp]
  return scalar by=_b[y: tfp]
  return scalar bi=_b[i: tfp]
 
end 

capture noisily simulate b_c = r(bc) b_y = r(by) b_i = r(bi) rmsey=r(rmsey) ///
rmsec=r(rmsec) rmsei=r(rmsei) seby=r(seby) sebc=r(sebc) sebi=r(sebi) sey=r(sey) ///
sec=r(sec) sei=r(sei) conv=r(conv), reps(1000): olssim, obs(2000)

















